
% WHAT IF...? OUTPUT LAYER

% FIG 2A panel 3

% PANEL 3 OF 3 SHOWING CURRENT DRIVEN BY A SINGLE PC. CURRENT DRIVEN BY 
% EACH SPIKE HAS A TIME CONSTANT OF 1.25 ms (BLUE), 2.5 ms (YELLOW), 5 ms 
% (PINK), AND 7.5 ms (GREY). PC RATE: 200 Hz.
% Simulate one-quantum-per-spike charge entry to the nulcear soma

% Create the shape of charge entry with a peak-and-tail signature of trans-
% membrane currrent, with an adjustable time constant. It is unnecessary to 
% also adjust peak amplitude because the functional relevance is through 
% the effect it has on the relative standard deviation of summed charge in 
% the functional range of PC rates. 

opengl software
figure;

x = -0.7:0.05:20;

g1_amp = .985; % amplitude - increasing the value increases peak amplitude.
% Changing the amplitude doesn't change the rise time. It is only used to
% compensate for an effect on amplitude of dur.
g1_dur = 2.5; % duration - increasing the value increases the time constant
% of skewedgaussian (by a diminishing ratio i.e. a linear increase 
% of time constant requires an exponential increase of dur).

gaussian1 = @(x) ((1/sqrt((2*pi))*exp(-x.^2/g1_dur))*g1_amp)% a normal distribution
skewedgaussian1 = @(x,alpha) 2*gaussian1(x).*normcdf(alpha*x)% that normal 
% distribution skewed

g2_amp = .92;  
g2_dur = 8.6; 

gaussian2 = @(x) ((1/sqrt((2*pi))*exp(-x.^2/g2_dur))*g2_amp)% the same 
% distribution as gaussian1 but with SD adjusted by SDproxy
skewedgaussian2 = @(x,alpha) 2*gaussian2(x).*normcdf(alpha*x)% skewed

g3_amp = .89;
g3_dur = 31;

gaussian3 = @(x) ((1/sqrt((2*pi))*exp(-x.^2/g3_dur))*g3_amp)
skewedgaussian3 = @(x,alpha) 2*gaussian3(x).*normcdf(alpha*x)

g4_amp = .88;
g4_dur = 66.5;

gaussian4 = @(x) ((1/sqrt((2*pi))*exp(-x.^2/g4_dur))*g4_amp)
skewedgaussian4 = @(x,alpha) 2*gaussian4(x).*normcdf(alpha*x)

% g5_amp = .88;
% g5_dur = 74;
% 
% gaussian5 = @(x) ((1/sqrt((2*pi))*exp(-x.^2/g5_dur))*g5_amp)
% skewedgaussian5 = @(x,alpha) 2*gaussian5(x).*normcdf(alpha*x)

% plot(x, skewedgaussian1(x, 4))
% hold on
% % plot(x, gaussian1(x))
% % plot(x, gaussian2(x))
% % plot(x, gaussian3(x))
% % plot(x, gaussian4(x))
% % plot(x, gaussian5(x))
% plot(x, skewedgaussian2(x, 4))
% plot(x, skewedgaussian3(x, 4))
% plot(x, skewedgaussian4(x, 4))
% % plot(x, skewedgaussian5(x, 4))

% DERIVE TIME CONSTANTS
g1_influxUnit = skewedgaussian1(x, 4); % an array of y values in the range for x, 
% set above, representing charge influx driven by a single spike
[M,I] = max(g1_influxUnit); % M is the peak value in array and I its index position
drop = M*.368; % a 63.2% reduction of M
[~,closestIndex] = min(abs(drop-g1_influxUnit(I:end))); % Finds the index position 
% of the nearest value to 'drop' in the array 'I:end'
g1_timeConstant = closestIndex/20 % There are 20 index positions per ms, so 
% dividing the distance (measured in index positions) between I and
% closestIndex by 20 gives the time interval between the peak value of
% the skewed distribution and a reduction by 63.2%

g2_influxUnit = skewedgaussian2(x, 4);
[M,I] = max(g2_influxUnit);
drop = M*.368;
[~,closestIndex] = min(abs(drop-g2_influxUnit(I:end)));
g2_timeConstant = closestIndex/20

g3_influxUnit = skewedgaussian3(x, 4);
[M,I] = max(g3_influxUnit);
drop = M*.368;
[~,closestIndex] = min(abs(drop-g3_influxUnit(I:end)));
g3_timeConstant = closestIndex/20

g4_influxUnit = skewedgaussian4(x, 4);
[M,I] = max(g4_influxUnit);
drop = M*.368;
[~,closestIndex] = min(abs(drop-g4_influxUnit(I:end)));
g4_timeConstant = closestIndex/20

% g5_influxUnit = skewedgaussian5(x, 4);
% [M,I] = max(g5_influxUnit);
% drop = M*.368;
% [~,closestIndex] = min(abs(drop-g5_influxUnit(I:end)));
% g5_timeConstant = closestIndex/20

% SUMMARY
% Create an array of 4,000 zeros representing a time interval of 200 ms
% divided into twentieths of a ms. An array represents a single PC. More 
% exactly, it will be used to represent charge influx driven by spike input 
% from a single PC. At each index position, there is a probability, P, of
% an input spike. At a steady afferent rate, it is the same in all moments. 
% When there is a spike, 0 is replaced by 0.001 at that index position.  
% Assume a 1 ms refectory period. Call the array PC1. Call the first such 
% index position index1. Call the next index2, and so on. Next, add 
% 'influx' to PC1 at index positions index1:index1+(length(influx)-1).
% Then, repeat at index positions index2:index2+(length(influx)-1), etc.

%%% 200 Hz %%%

Hz = 200; % PC firing rate
p = Hz/20000; % The probability of a spike in any moment (a moment being 
% 1/20th of a ms.

% g1 PLOT
PCspikes = zeros([1 4400]);

for i = 1:4400
    r = rand;
    if r <= p
        PCspikes(i) = 1;
    end
end

PCcurrent = zeros([1 4400]);

i = 1;
while i < 4400
    if i < length(PCspikes)-(length(g1_influxUnit)-1)
        if PCspikes(i) == 1
            PCcurrent(i:i+length(g1_influxUnit)-1) = ...
                PCcurrent(i:i+length(g1_influxUnit)-1)+g1_influxUnit;
            i = i + 20;
        else
            i = i + 1;
        end
    else
        i = i + 1;
    end
end

subplot(6,1,[1 2])
x = 1:4400;
plot(x,PCcurrent,'Color',([0 0 .6]),'LineWidth',1.4)
xlabel(''); ylabel('current'); title('1 PC, 200 Hz, x = ms');
xlim([0 4200]); xticks([0 500 1000 1500 2000 2500 3000 3500 4000]);
xticklabels([0 25 50 75 100 125 150 175 200]);
ylim([0 2.8]); yticks([0 0.7 1.4 2.1 2.8]);
yticklabels([0 1 2 3 4])
set(gca,'FontSize',10)

hold on

% g2 PLOT
PCspikes = zeros([1 4400]);

for i = 1:4400
    r = rand;
    if r <= p
        PCspikes(i) = 1;
    end
end

PCcurrent = zeros([1 4400]);

i = 1;
while i < 4400
    if i < length(PCspikes)-(length(g2_influxUnit)-1)
        if PCspikes(i) == 1
            PCcurrent(i:i+length(g2_influxUnit)-1) = ...
                PCcurrent(i:i+length(g2_influxUnit)-1)+g2_influxUnit;
            i = i + 20;
        else
            i = i + 1;
        end
    else
        i = i + 1;
    end
end

plot(x,PCcurrent,'Color',([1 .84 0]),'LineWidth',1.4)

% g3 PLOT
PCspikes = zeros([1 4400]);

for i = 1:4400
    r = rand;
    if r <= p
        PCspikes(i) = 1;
    end
end

PCcurrent = zeros([1 4400]);

i = 1;
while i < 4400
    if i < length(PCspikes)-(length(g3_influxUnit)-1)
        if PCspikes(i) == 1
            PCcurrent(i:i+length(g3_influxUnit)-1) = ...
                PCcurrent(i:i+length(g3_influxUnit)-1)+g3_influxUnit;
            i = i + 20;
        else
            i = i + 1;
        end
    else
        i = i + 1;
    end
end

plot(x,PCcurrent,'Color',([1 .5 .5]),'LineWidth',1.4)

% g4 PLOT
PCspikes = zeros([1 4400]);

for i = 1:4400
    r = rand;
    if r <= p
        PCspikes(i) = 1;
    end
end

PCcurrent = zeros([1 4400]);

i = 1;
while i < 4400
    if i < length(PCspikes)-(length(g4_influxUnit)-1)
        if PCspikes(i) == 1
            PCcurrent(i:i+length(g4_influxUnit)-1) = ...
                PCcurrent(i:i+length(g4_influxUnit)-1)+g4_influxUnit;
            i = i + 20;
        else
            i = i + 1;
        end
    else
        i = i + 1;
    end
end

plot(x,PCcurrent,'Color',([.6 .6 .6]),'LineWidth',1.4)
